package com.mcwl.resource.handler.impl;

import cn.hutool.json.JSON;
import cn.hutool.json.JSONUtil;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.exception.ServiceException;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.StringUtils;
import com.mcwl.resource.domain.ModelImageComment;
import com.mcwl.resource.domain.ModelImageCommentLike;
import com.mcwl.resource.domain.SysAdvice;
import com.mcwl.resource.handler.IMessageHandler;
import com.mcwl.resource.mapper.ModelImageCommentLikeMapper;
import com.mcwl.resource.mapper.ModelImageCommentMapper;
import com.mcwl.resource.mapper.WorkFlowCommentLikeMapper;
import com.mcwl.resource.mapper.WorkFlowCommentMapper;
import com.mcwl.resource.service.ISysAdviceService;
import com.mcwl.system.domain.enums.AdviceEnum;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Map;
import java.util.Objects;

@Slf4j
@Component
@IMessageHandler.Topic(value = "imageCommentLike/#", qos = 1)
@RequiredArgsConstructor
public class ImageCommentLikeHandler implements IMessageHandler {

    private final ModelImageCommentMapper modelImageCommentMapper;

    private final ModelImageCommentLikeMapper modelImageCommentLikeMapper;

    private final ISysUserService userService;

    private final ISysAdviceService adviceService;
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void handleMessage(String topic, MqttMessage message) {
        String payload = new String(message.getPayload());
        log.info("图片评论点赞: {} - {}", payload, message);

        JSON parse = JSONUtil.parse(payload);
        Long commentId = Long.parseLong(parse.getByPath("commentId").toString());
        Long userId = Long.parseLong(parse.getByPath("userId").toString());



        this.like(userId, commentId);
    }



    private void  like(Long userId, Long commentId) {
        ModelImageComment modelImageComment = modelImageCommentMapper.selectById(commentId);
        if (Objects.isNull(modelImageComment)) {
            throw new ServiceException("该评论不存在");
        }
        ModelImageCommentLike modelImageCommentLike = modelImageCommentLikeMapper.getLikeImageComment(userId, commentId);
        if (Objects.nonNull(modelImageCommentLike)) {
            if (Objects.equals(modelImageCommentLike.getDelFlag(), "0")) {
                modelImageCommentLike.setDelFlag("2");
                int likeNum = modelImageComment.getLikeNum() - 1;
                likeNum = Math.max(likeNum, 0);
                modelImageComment.setLikeNum(likeNum);
            } else {
                modelImageCommentLike.setDelFlag("0");
                modelImageComment.setLikeNum(modelImageComment.getLikeNum() + 1);
                this.addLikeAdvice(modelImageComment, userId);
            }
            // 更新点赞记录
            modelImageCommentLikeMapper.updateDelFlagById(modelImageCommentLike);
            // 更新图片评论点赞数
            modelImageCommentMapper.updateById(modelImageComment);
            return;
        }

        // 添加点赞记录
        modelImageCommentLike = new ModelImageCommentLike();
        modelImageCommentLike.setUserId(userId);
        modelImageCommentLike.setModelImageCommentId(commentId);
        modelImageCommentLikeMapper.insert(modelImageCommentLike);

        // 更新图片点赞数
        modelImageComment.setLikeNum(modelImageComment.getLikeNum() + 1);
        modelImageCommentMapper.updateById(modelImageComment);

        this.addLikeAdvice(modelImageComment, userId);
    }


    private void addLikeAdvice(ModelImageComment modelImageComment, Long userId) {

        Long receiverUserId = modelImageComment.getUserId();

        if (Objects.equals(userId, receiverUserId)) {
            return;
        }

        SysUser sysUser = userService.selectUserById(userId);

        String content = StringUtils.format("恭喜！{}点赞了您的评论",
                sysUser.getNickName());

        SysAdvice sysAdvice = new SysAdvice();
        sysAdvice.setSenderId(userId);
        sysAdvice.setReceiverId(receiverUserId);
        sysAdvice.setContent(content);
        sysAdvice.setProductId(modelImageComment.getModelImageId());
        sysAdvice.setProductType(2);
        sysAdvice.setCommentId(modelImageComment.getId());
        sysAdvice.setIsRead(0);
        sysAdvice.setType(AdviceEnum.LIKE_REMIND);
        adviceService.save(sysAdvice);


    }



}