package com.mcwl.resource.handler.impl;

import cn.hutool.json.JSON;
import cn.hutool.json.JSONUtil;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.exception.ServiceException;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.StringUtils;
import com.mcwl.resource.domain.*;
import com.mcwl.resource.handler.IMessageHandler;
import com.mcwl.resource.mapper.ModelCommentLikeMapper;
import com.mcwl.resource.mapper.ModelCommentMapper;
import com.mcwl.resource.mapper.WorkFlowCommentLikeMapper;
import com.mcwl.resource.mapper.WorkFlowCommentMapper;
import com.mcwl.resource.service.ISysAdviceService;
import com.mcwl.system.domain.enums.AdviceEnum;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Objects;

@Slf4j
@Component
@IMessageHandler.Topic(value = "modelCommentLike/#", qos = 1)
@RequiredArgsConstructor
public class ModelCommentLikeHandler implements IMessageHandler {


    private final ModelCommentMapper modelCommentMapper;

    private final ModelCommentLikeMapper modelCommentLikeMapper;

    private final ISysUserService userService;

    private final ISysAdviceService adviceService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void handleMessage(String topic, MqttMessage message) {
        String payload = new String(message.getPayload());
        log.info("模型评论点赞: {} - {}", payload, message);

        JSON parse = JSONUtil.parse(payload);
        Long commentId = Long.parseLong(parse.getByPath("commentId").toString());
        Long userId = Long.parseLong(parse.getByPath("userId").toString());


        this.like(userId, commentId);
    }

    private void like(Long userId, Long commentId) {
        ModelComment modelComment = modelCommentMapper.selectById(commentId);
        if (Objects.isNull(modelComment)) {
            throw new ServiceException("该评论不存在");
        }
        ModelCommentLike modelCommentLike = modelCommentLikeMapper.getLikeComment(userId, commentId);
        if (Objects.nonNull(modelCommentLike)) {
            if (Objects.equals(modelCommentLike.getDelFlag(), "0")) {
                modelCommentLike.setDelFlag("2");
                int likeNum = modelComment.getLikeNum() - 1;
                likeNum = Math.max(likeNum, 0);
                modelComment.setLikeNum(likeNum);
            } else {
                modelCommentLike.setDelFlag("0");
                modelComment.setLikeNum(modelComment.getLikeNum() + 1);
                this.addLikeAdvice(modelComment, userId);
            }
            // 更新点赞记录
            modelCommentLikeMapper.updateDelFlagById(modelCommentLike);
            // 更新图片评论点赞数
            modelCommentMapper.updateById(modelComment);
            return;
        }

        // 添加点赞记录
        modelCommentLike = new ModelCommentLike();
        modelCommentLike.setUserId(userId);
        modelCommentLike.setModelCommentId(commentId);
        modelCommentLikeMapper.insert(modelCommentLike);

        // 更新模型点赞数
        modelComment.setLikeNum(modelComment.getLikeNum() + 1);
        modelCommentMapper.updateById(modelComment);

        this.addLikeAdvice(modelComment, userId);
    }


    private void addLikeAdvice(ModelComment modelComment, Long userId) {

        Long receiverUserId = modelComment.getUserId();

        if (Objects.equals(userId, receiverUserId)) {
            return;
        }
        SysUser sysUser = userService.selectUserById(userId);

        String content = StringUtils.format("恭喜！{}点赞了您的评论",
                sysUser.getNickName());

        SysAdvice sysAdvice = new SysAdvice();
        sysAdvice.setSenderId(userId);
        sysAdvice.setReceiverId(receiverUserId);
        sysAdvice.setContent(content);
        sysAdvice.setProductId(modelComment.getModelId());
        sysAdvice.setProductType(0);
        sysAdvice.setCommentId(modelComment.getId());
        sysAdvice.setIsRead(0);
        sysAdvice.setType(AdviceEnum.LIKE_REMIND);
        adviceService.save(sysAdvice);


    }



}