package com.mcwl.resource.handler.impl;

import cn.hutool.json.JSON;
import cn.hutool.json.JSONUtil;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.exception.ServiceException;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.StringUtils;
import com.mcwl.resource.domain.*;
import com.mcwl.resource.handler.IMessageHandler;
import com.mcwl.resource.mapper.ModelLikeMapper;
import com.mcwl.resource.mapper.ModelMapper;
import com.mcwl.resource.mapper.WorkFlowLikeMapper;
import com.mcwl.resource.mapper.WorkFlowMapper;
import com.mcwl.resource.service.ISysAdviceService;
import com.mcwl.system.domain.enums.AdviceEnum;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Objects;

@Slf4j
@Component
@IMessageHandler.Topic(value = "modelLike/#", qos = 1)
@RequiredArgsConstructor
public class ModelLikeHandler implements IMessageHandler {

    private final ModelMapper modelMapper;


    private final ModelLikeMapper modelLikeMapper;

    private final ISysUserService userService;

    private final ISysAdviceService adviceService;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void handleMessage(String topic, MqttMessage message) {
        String payload = new String(message.getPayload());
        log.info("模型点赞: {}", payload);

        JSON parse = JSONUtil.parse(payload);
        Long commentId = Long.parseLong(parse.getByPath("commentId").toString());
        Long userId = Long.parseLong(parse.getByPath("userId").toString());


        this.like(userId, commentId);
    }

    private void like(Long userId, Long modelId) {
        ModelProduct model = modelMapper.selectById(modelId);
        if (Objects.isNull(model)) {
            throw new ServiceException("该模型不存在或已下架");
        }
        ModelLike modelLike = modelLikeMapper.getLike(userId, modelId);
        if (Objects.nonNull(modelLike)) {
            if (Objects.equals(modelLike.getDelFlag(), "0")) {
                modelLike.setDelFlag("2");
                int likeNum = model.getLikeNum() - 1;
                likeNum = Math.max(likeNum, 0);
                model.setLikeNum(likeNum);
            } else {
                modelLike.setDelFlag("0");
                model.setLikeNum(model.getLikeNum() + 1);
                this.addLikeAdvice(model, userId);
            }
            // 更新点赞记录
            modelLikeMapper.updateDelFlagById(modelLike);
            // 更新图片点赞数
            modelMapper.updateById(model);
            return;
        }

        // 添加点赞记录
        modelLike = new ModelLike();
        modelLike.setUserId(userId);
        modelLike.setModelId(modelId);
        modelLikeMapper.insert(modelLike);

        // 更新图片点赞数
        model.setNumbers(model.getLikeNum() + 1);
        modelMapper.updateById(model);

        this.addLikeAdvice(model, userId);
    }


    private void addLikeAdvice(ModelProduct model, Long userId) {

        Long receiverUserId = model.getUserId();

        if (Objects.equals(userId, receiverUserId)) {
            return;
        }
        SysUser sysUser = userService.selectUserById(userId);

        String content = StringUtils.format("恭喜！{}点赞了您的模型：{}",
                sysUser.getNickName(), model.getModelName());

        SysAdvice sysAdvice = new SysAdvice();
        sysAdvice.setSenderId(userId);
        sysAdvice.setReceiverId(receiverUserId);
        sysAdvice.setContent(content);
        sysAdvice.setProductId(model.getId());
        sysAdvice.setProductType(0);
        sysAdvice.setIsRead(0);
        sysAdvice.setType(AdviceEnum.LIKE_REMIND);
        adviceService.save(sysAdvice);


    }


}