package com.mcwl.resource.handler.impl;

import cn.hutool.json.JSON;
import cn.hutool.json.JSONUtil;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.exception.ServiceException;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.StringUtils;
import com.mcwl.resource.domain.SysAdvice;
import com.mcwl.resource.domain.WorkFlow;
import com.mcwl.resource.domain.WorkFlowLike;
import com.mcwl.resource.handler.IMessageHandler;
import com.mcwl.resource.mapper.WorkFlowLikeMapper;
import com.mcwl.resource.mapper.WorkFlowMapper;
import com.mcwl.resource.service.ISysAdviceService;
import com.mcwl.system.domain.enums.AdviceEnum;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Objects;

@Slf4j
@Component
@IMessageHandler.Topic(value = "workFlowLike/#", qos = 1)
@RequiredArgsConstructor
public class WorkFlowLikeHandler implements IMessageHandler {

    private final WorkFlowMapper workFlowMapper;

    private final WorkFlowLikeMapper workFlowLikeMapper;

    private final ISysUserService userService;

    private final ISysAdviceService adviceService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void handleMessage(String topic, MqttMessage message) {
        String payload = new String(message.getPayload());
        log.info("工作流点赞: {} - {}", topic, payload);

        JSON parse = JSONUtil.parse(payload);
        Long commentId = Long.parseLong(parse.getByPath("commentId").toString());
        Long userId = Long.parseLong(parse.getByPath("userId").toString());


        this.like(userId, commentId);
    }

    private void like(Long userId, Long modelId) {
        WorkFlow workFlow = workFlowMapper.selectById(modelId);
        if (Objects.isNull(workFlow)) {
            throw new ServiceException("该工作流不存在或已下架");
        }
        WorkFlowLike workFlowLike = workFlowLikeMapper.getLike(userId, modelId);
        if (Objects.nonNull(workFlowLike)) {
            if (Objects.equals(workFlowLike.getDelFlag(), "0")) {
                workFlowLike.setDelFlag("2");
                int likeCount = workFlow.getLikeNum() - 1;
                likeCount = Math.max(likeCount, 0);
                workFlow.setLikeNum(likeCount);
            } else {
                workFlowLike.setDelFlag("0");
                workFlow.setLikeNum(workFlow.getLikeNum() + 1);
                this.addLikeAdvice(workFlow, userId);
            }
            // 更新点赞记录
            workFlowLikeMapper.updateStatus(workFlowLike);
            // 更新图片点赞数
            workFlowMapper.updateById(workFlow);
            return;
        }

        // 添加点赞记录
        workFlowLike = new WorkFlowLike();
        workFlowLike.setUserId(userId);
        workFlowLike.setWorkFlowId(modelId);
        workFlowLikeMapper.insert(workFlowLike);

        // 更新图片点赞数
        workFlow.setLikeNum(workFlow.getLikeNum() + 1);
        workFlowMapper.updateById(workFlow);

        this.addLikeAdvice(workFlow, userId);
    }


    private void addLikeAdvice(WorkFlow workFlow, Long userId) {

        Long receiverUserId = workFlow.getUserId();

        if (Objects.equals(userId, receiverUserId)) {
            return;
        }
        SysUser sysUser = userService.selectUserById(userId);

        String content = StringUtils.format("恭喜！{}点赞了您的工作流：{}",
                sysUser.getNickName(), workFlow.getWorkflowName());

        SysAdvice sysAdvice = new SysAdvice();
        sysAdvice.setSenderId(userId);
        sysAdvice.setReceiverId(receiverUserId);
        sysAdvice.setContent(content);
        sysAdvice.setProductId(workFlow.getId());
        sysAdvice.setProductType(1);
        sysAdvice.setIsRead(0);
        sysAdvice.setType(AdviceEnum.LIKE_REMIND);
        adviceService.save(sysAdvice);


    }


}