package com.mcwl.resource.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.constant.HttpStatus;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.core.page.PageDomain;
import com.mcwl.common.core.page.TableDataInfo;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.StringUtils;
import com.mcwl.resource.domain.ModelLike;
import com.mcwl.resource.domain.ModelProduct;
import com.mcwl.resource.domain.vo.ModelLikeVo;
import com.mcwl.resource.mapper.ModelLikeMapper;
import com.mcwl.resource.mapper.ModelMapper;
import com.mcwl.resource.service.ModelLikeService;
import com.mcwl.resource.util.MqttTemplate;
import com.mcwl.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 模型点赞
 */
@Service
public class ModelLikeServiceImpl extends ServiceImpl<ModelLikeMapper, ModelLike> implements ModelLikeService {


    @Autowired
    private ModelMapper modelMapper;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private MqttTemplate mqttTemplate;


    @Override
    public void like(Long modelId) {
        Map<String, Long> map = new HashMap<>();
        map.put("userId", SecurityUtils.getUserId());
        map.put("commentId", modelId);
        mqttTemplate.publish("modelLike", JSONUtil.toJsonStr(map),0);
    }

    @Override
    public TableDataInfo listByPage(PageDomain pageDomain) {
        Page<ModelLike> page = initPage(pageDomain);


        baseMapper.selectPage(page, new LambdaQueryWrapper<ModelLike>()
                .eq(ModelLike::getUserId, SecurityUtils.getUserId()));

        List<ModelLike> modelLikeList = page.getRecords();
        List<ModelLikeVo> modelLikeVoList = new ArrayList<>();
        for (ModelLike modelLike : modelLikeList) {
            ModelLikeVo modelLikeVo = new ModelLikeVo();
            BeanUtil.copyProperties(modelLike, modelLikeVo);

            // 获取用户信息
            SysUser sysUser = sysUserService.selectUserById(modelLike.getUserId());
            if (Objects.isNull(sysUser)) {
                page.setTotal(page.getTotal() - 1);
                continue;
            }
            modelLikeVo.setUserName(sysUser.getNickName());
            modelLikeVo.setUserAvatar(sysUser.getAvatar());

            // 获取模型信息
            ModelProduct modelProduct = modelMapper.selectById(modelLike.getModelId());
            if (Objects.isNull(modelProduct)) {
                page.setTotal(page.getTotal() - 1);
                continue;
            }
            modelLikeVo.setDownloadNumber(Long.valueOf(modelProduct.getNumbers()));
            modelLikeVo.setSurfaceUrl(modelProduct.getSurfaceUrl());
            modelLikeVo.setModelName(modelProduct.getModelName());

            modelLikeVoList.add(modelLikeVo);
        }


        TableDataInfo tableDataInfo = new TableDataInfo();
        tableDataInfo.setRows(modelLikeVoList);
        tableDataInfo.setTotal(page.getTotal());
        tableDataInfo.setCode(HttpStatus.SUCCESS);
        tableDataInfo.setMsg("查询成功");

        return tableDataInfo;


    }


    private Page<ModelLike> initPage(PageDomain pageDomain) {
        Page<ModelLike> page = new Page<>(pageDomain.getPageNum(), pageDomain.getPageSize());
        if (StringUtils.isBlank(pageDomain.getOrderByColumn())) {
            pageDomain.setOrderByColumn("create_time");
        }
        OrderItem orderItem = OrderItem.desc(pageDomain.getOrderByColumn());
        page.addOrder(orderItem);

        return page;
    }


}
